import os
import sys
from pathlib import Path


class Config(object):
    class Forced:
        def __init__(self, config):
            self.config = config
            self.force = config.force

        def __enter__(self):
            self.config.force = True
            return self.config

        def __exit__(self, type, value, traceback):
            self.config.force = self.force

    def __init__(self, quiet=False, project_name=None, force=False, verbose=False):
        from .consts import YAML_VERSION
        from . import odoo_config  # NOQA

        from .init_functions import _get_customs_root

        self.WORKING_DIR = _get_customs_root(Path(os.getcwd()))
        self._host_run_dir = None
        self.project_name = project_name
        self.YAML_VERSION = YAML_VERSION
        self.verbose = verbose
        self.force = force
        self.compose_version = YAML_VERSION
        self.quiet = quiet
        self.restrict = []

    @property
    def project_name(self):
        return self._project_name

    @project_name.setter
    def project_name(self, value):
        self._project_name = value
        if self._project_name:
            os.environ["PROJECT_NAME"] = value
            self.HOST_RUN_DIR = Path(os.environ["HOME"]) / ".odoo" / "run" / value
        else:
            os.environ["PROJECT_NAME"] = ""
        self.setup_files_and_folders()

    @property
    def HOST_RUN_DIR(self):
        return self._host_run_dir

    @HOST_RUN_DIR.setter
    def HOST_RUN_DIR(self, value):
        self._host_run_dir = value
        if value:
            os.environ["HOST_RUN_DIR"] = str(value)
        self.setup_files_and_folders()

    def setup_files_and_folders(self):
        from .init_functions import get_use_docker
        from . import odoo_config  # NOQA

        self.dirs = {}
        self.files = {}
        self.commands = {}

        self.use_docker = get_use_docker(self.files)
        from .init_functions import make_absolute_paths

        make_absolute_paths(self, self.dirs, self.files, self.commands)

    def forced(self):
        return Config.Forced(self)

    def __getattribute__(self, name):
        try:
            value = super(Config, self).__getattribute__(name)
            return value
        except AttributeError:
            from .myconfigparser import MyConfigParser  # NOQA

            if "settings" not in self.files:
                return None
            myconfig = MyConfigParser(self.files["settings"])

            convert = None
            if name.endswith("_as_int"):
                convert = "asint"
                name = name[: -len("_as_int")]
            elif name.endswith("_as_bool"):
                convert = "asbool"
                name = name[: -len("_as_bool")]

            for tries in [name, name.lower(), name.upper()]:
                value = ""
                if tries not in myconfig.keys():
                    continue

                value = myconfig.get(tries, "")
                break

            if convert:
                if convert == "asint":
                    value = int(value or "0")

            if value == "1":
                value = True
            elif value == "0":
                value = False
            return value
        except Exception:
            raise

    def get_odoo_conn(self, inside_container=None):
        from .odoo_config import get_postgres_connection_params  # NOQA
        from .tools import DBConnection

        host, port, user, password = get_postgres_connection_params(
            inside_container=inside_container
        )
        conn = DBConnection(self.dbname, host, port, user, password)
        return conn



import os
import tempfile
import click
from pathlib import Path
from .odoo_config import current_version
from .odoo_config import customs_dir
from . import cli, pass_config, Commands
from .lib_clickhelpers import AliasedGroup
import subprocess

@cli.group(cls=AliasedGroup)
@pass_config
def control(config):
    pass

@control.command()
@pass_config
@click.pass_context
def dev(ctx, config):
    """
    starts developing in the odoo container
    """
    from .myconfigparser import MyConfigParser
    myconfig = MyConfigParser(config.files['settings'])
    proxy_port = myconfig['PROXY_PORT']
    roundcube_port = myconfig['ROUNDCUBE_PORT']
    ip = '127.0.0.1'
    click.secho("Proxy Port: http://{}:{}".format(ip, proxy_port), fg='green', bold=True)
    click.secho("Mailclient : http://{}:{}".format(ip, roundcube_port), fg='green', bold=True)
    _exec_in_virtualenv(config, ['python', _path_odoolib() / 'debug.py'])


def _update_command(config, params):
    _exec_in_virtualenv(config, ['python', _path_odoolib() / 'update_modules.py'] + params)


def up(ctx, config, machines=[], daemon=False, remove_orphans=True):
    _exec_in_virtualenv(config, ['python', _path_odoolib() / 'run.py'])


@control.command()
def shell(config):
    subprocess.call([
        'python',
        ''
    ])
    _exec_in_virtualenv(config, ['python', _path_odoolib() / 'shell.py'])

def _path_odoolib():
    return Path(os.environ['ODOO_HOME']) / 'images' / 'odoo' / 'bin'

def _exec_in_virtualenv(config, cmd):
    filename = Path(tempfile.mktemp(suffix='.sh'))
    from .myconfigparser import MyConfigParser
    myconfig = MyConfigParser(config.files['settings'])

    def _quoted(x):
        return "'{}'".format(x)
    content = []
    for key in myconfig.keys():
        content.append("export {}='{}'".format(key, myconfig.get(key)))
    files = config.files
    dirs = config.dirs

    raise Exception('adapt after pip compatible')

    filename.write_text('\n'.join(content + list((
        f'set -ex',
        f'export DEBUGGER_WATCH="{files["run/odoo_debug.txt"]}"',
        f'export ODOO_CONFIG_TEMPLATE_DIR="{dirs["images"] / "odoo" / "config" / str(current_version()) / "config"}"',
        f'export ODOO_CONFIG_DIR="{dirs["run_native_config_dir"]}"',
        f'export ODOOLIB="{dirs["images"] / "odoo" / "bin"}"',
        f'export ODOO_USER="$(whoami)"',
        f'export ODOO_DATA_DIR="{dirs["odoo_data_dir"]}"',
        f'export SERVER_DIR="{customs_dir()}/odoo"',
        f'export PYTHONPATH="{dirs["."]}"',
        f'export RUN_DIR="{dirs["run"]}"',
        f'export NO_SOFFICE=1',
        f'export OUT_DIR="{dirs["run_native_out_dir"]}"',
        f'export INTERNAL_ODOO_PORT=8069',
        f"source '{dirs['venv']}/bin/activate'",
        f"which python",
        f"python --version",
        f"{' '.join(map(_quoted, cmd))}",
    ))))

    subprocess.call(["/bin/bash", filename])
    print("--------------------------------------")
    # print(filename.read_text())
    # filename.unlink()
